﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet
{
    /// <summary>
    /// Represents a randomiser that use the pseudo-random number generator provided by
    /// <see cref="Random"/>.
    /// </summary>
    /// <remarks>
    /// See the <b>Remarks</b> section of <see cref="Random"/> for more information on the algorithm
    /// used to generate the random numbers.
    /// </remarks>
    public class BasicRandomiser : IRandomiser
    {
        private Random rng; // Pseudo-random number generator.

        /// <summary>
        /// Initialises a new instance of the <see cref="BasicRandomiser"/> class, using the
        /// specified seed value.
        /// </summary>
        /// <param name="seed"> A number used to calculate a starting value for the pseudo-random
        /// number sequence. If a negative number is specified, the absolute value of the number
        /// is used.</param>
        public BasicRandomiser(int seed)
        {
            this.rng = new Random(seed);
        }

        /// <summary>
        /// Initialises a new instance of the <see cref="BasicRandomiser"/> class.
        /// </summary>
        public BasicRandomiser()
        {
            this.rng = new Random();
        }

        /// <inheritdoc/>
        public int GetInteger(int minValue, int maxValue)
        {
            return this.rng.Next(minValue, maxValue);
        }

        /// <inheritdoc/>
        public int GetInteger(int maxValue)
        {
            return this.rng.Next(maxValue);
        }

        /// <inheritdoc/>
        public int GetInteger()
        {
            return this.rng.Next();
        }

        /// <inheritdoc/>
        public double GetDouble()
        {
            return this.rng.NextDouble();
        }
    }
}
