﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DarwinDotNet
{
    /// <summary>
    /// Represents a device that can generate (pseudo-)random numbers (floating-point values and
    /// integers).
    /// </summary>
    public interface IRandomiser
    {
        /// <summary>
        /// Returns a non-negative random number within a specified range.
        /// </summary>
        /// <param name="minValue">The inclusive lower bound of the random number returned.</param>
        /// <param name="maxValue">The exclusive upper bound of the random number to be generated.
        /// The value must be greater than or equal to zero.</param>
        /// <returns> A 32-bit signed integer greater than or equal to <paramref name="minValue"/>
        /// and less than <paramref name="maxValue"/>; that is, the range of return values includes
        /// <paramref name="minValue"/> but not <paramref name="maxValue"/>. If
        /// <paramref name="minValue"/> equals <paramref name="maxValue"/>, 
        /// <paramref name="minValue"/> is returned.</returns>
        int GetInteger(int minValue, int maxValue);
        
        /// <summary>
        /// Returns a non-negative random number less than the specified maximum.
        /// </summary>
        /// <param name="maxValue">The exclusive upper bound of the random number to be generated.
        /// The value must be greater than or equal to zero.</param>
        /// <returns>A 32-bit signed integer greater than or equal to zero, and less than 
        /// <paramref name="maxValue"/>; that is, the range of return values ordinarily includes
        /// zero but not <paramref name="maxValue"/>. However, if <paramref name="maxValue"/> equals
        /// zero, <paramref name="maxValue"/> is returned.</returns>
        int GetInteger(int maxValue);

        /// <overloads>Returns a non-negative random number, optionally specifying the range.</overloads>
        /// <summary>
        /// Returns a non-negative random number.
        /// </summary>
        /// <returns>A 32-bit signed integer greater than or equal to zero and less than
        /// <see cref="System.Int32.MaxValue"/>.</returns>
        int GetInteger();
        
        /// <summary>
        /// Returns a random number between 0.0 and 1.0.
        /// </summary>
        /// <returns>A double-precision floating point number greater than or equal to 0.0, and less
        /// than 1.0.</returns>
        double GetDouble();
    }
}
