﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DarwinDotNet;

namespace HelloWorld
{
    public class SimpleStringMutator : Mutator<int>
    {
        public SimpleStringMutator()
            : base()
        {
            this.ProbabilityFunction = new Func<Population, double>(pop => 0.020d);
        }

        public override int Mutate(Chromosome<int> chromosome, int index, int codon)
        {
            return Randomiser.GetInteger(StringChromosome.Chars.Length);
        }
    }

    public class SimpleStringRecombinator : Recombinator
    {
        public SimpleStringRecombinator()
            : base()
        {
            this.ProbabilityFunction = new Func<Population, double>(pop => 1.00d);
        }

        public override void Recombine(Chromosome chromosomeA, Chromosome chromosomeB,
            out int index, out int length)
        {
            var minLength = Math.Min(chromosomeA.Length, chromosomeB.Length);

            // Double-point crossover.
            length = Randomiser.GetInteger(1, minLength);
            index = Randomiser.GetInteger(0, minLength - length);
        }
    }

    public class StringChromosome : Chromosome<int>
    {
        public const string Chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ :;',.?!";

        public StringChromosome(int length)
            : base(length)
        {
        }

        protected StringChromosome()
            : base()
        {
        }

        public override Chromosome<int> Clone(bool copyIsOperable)
        {
            var clone = new StringChromosome();
            CloneBase(clone, copyIsOperable);
            return clone;
        }

        public override string ToString()
        {
            return new string(Array.ConvertAll(this.Codons, item => (int)item < Chars.Length ?
                Chars[(int)item] : '?'));
        }
    }
}
