﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using DarwinDotNet;
using DarwinDotNet.Selectors;

namespace HelloWorld
{
    internal static class Program
    {
        private static readonly string target = "Hello world!";
        private static int[] targetChars;
        private static double maxFitness = (StringChromosome.Chars.Length - 1) * target.Length;

        private static IRandomiser randomiser = new BasicRandomiser();

        private static void Main(string[] args)
        {
            // Create array of target characters from target string.
            targetChars = new int[target.Length];
            for (int i = 0; i < target.Length; i++)
                targetChars[i] = StringChromosome.Chars.IndexOf(target[i]);

            // Create initial population and then evolve it.
            var population = new Population(2048);

            population.GeneticOperators = new Operator[] { 
                new ElitistSelector(205),
                new FitnessProportionalSelector(),
                //new TournamentSelector(20),
                //new RankSelector(),
                new SimpleStringMutator(),
                new SimpleStringRecombinator()
            };
            population.SetRandomiser(new BasicRandomiser());
            population.Generate(GenerateRandomChromosome);
            population.Evolve(RunSimulation, maxFitness);

            // Print out results.
            Console.WriteLine("Found solution in generation {0:#0}.", population.GenerationNumber);

#if DEBUG
            Console.ReadKey(true);
#endif
        }

        public static void RunSimulation(Population population)
        {
            var chromosomes = population.Chromosomes;

            //for (int i = 0; i < chromosomes.Count; i++)
            Parallel.For(0, chromosomes.Count, delegate(int i)
            {
                var chromosome = chromosomes[i];

                int sum = 0;
                for (int j = 0; j < chromosome.Length; j++)
                    sum += Math.Abs((int)chromosome[j] - targetChars[j]);
                chromosome.Fitness = maxFitness - sum;
            });

            Console.WriteLine("Generation #{0:#00000} | Best Fitness: {1:#00000} | " +
                "Average Fitness: {2:#000000}",
                population.GenerationNumber, population.Chromosomes.Max(c => c.Fitness),
                population.Chromosomes.Average(c => c.Fitness));
        }

        public static Chromosome GenerateRandomChromosome(Population population)
        {
            var chromosome = new StringChromosome(target.Length);
            for (int i = 0; i < chromosome.Length; i++)
                chromosome[i] = randomiser.GetInteger(StringChromosome.Chars.Length);

            return chromosome;
        }
    }
}
