﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DarwinDotNet;
using DarwinDotNet.Expressions;

namespace SymbolicRegression
{
    public class SimpleExpressionMutator : Mutator<MathSymbol>
    {
        public SimpleExpressionMutator()
            : base()
        {
        }

        public override MathSymbol Mutate(Chromosome<MathSymbol> chromosome, int index, 
            MathSymbol codon)
        {
            var exprChromsome = (ExpressionChromosome<MathSymbol>)chromosome;
            if (index < exprChromsome.HeadLength)
                return (MathSymbol)Randomiser.GetInteger(0, (int)MathSymbol.Max);
            else
                return (MathSymbol)Randomiser.GetInteger((int)MathSymbol.FunctionMax, 
                    (int)MathSymbol.Max);
        }
    }

    public class SimpleExpressionTranspositor : Transpositor
    {
        public SimpleExpressionTranspositor()
            : base()
        {
        }

        public override void Transpose(Chromosome chromosome, out int startIndex, 
            out int destinationIndex, out int length, out bool insert)
        {
            length = Randomiser.GetInteger(1, 4);
            startIndex = Randomiser.GetInteger(0, chromosome.Length - length);
            destinationIndex = Randomiser.GetInteger(1, chromosome.Length - length);
            insert = false;
        }
    }

    public class SimpleExpressionRecombinator : Recombinator
    {
        public SimpleExpressionRecombinator()
            : base()
        {
        }

        public override void Recombine(Chromosome chromosomeA, Chromosome chromosomeB,
            out int index, out int length)
        {
            // Use double-point crossover.
            var minLength = Math.Min(chromosomeA.Length, chromosomeB.Length);

            length = Randomiser.GetInteger(1, minLength);
            index = Randomiser.GetInteger(0, minLength - length);
        }
    }

    //public class MathExpression : Expression<MathSymbol>
    //{
    //    public MathExpression(MathSymbol symbol, Expression<MathSymbol> parent, 
    //        int childrenCapacity)
    //        : base(symbol, parent, childrenCapacity)
    //    {
    //    }

    //    public MathExpression()
    //        : base()
    //    {
    //    }

    //    public double Evaluate(double x, double y, double z)
    //    {
    //        if (!this.IsComplete)
    //            return double.NaN;

    //        switch (this.Symbol)
    //        {
    //            case MathSymbol.Add:
    //                return ((MathExpression)this.Children[0]).Evaluate(x, y, z) +
    //                    ((MathExpression)this.Children[1]).Evaluate(x, y, z);
    //            case MathSymbol.Subtract:
    //                return ((MathExpression)this.Children[0]).Evaluate(x, y, z) -
    //                    ((MathExpression)this.Children[1]).Evaluate(x, y, z);
    //            case MathSymbol.Multiply:
    //                return ((MathExpression)this.Children[0]).Evaluate(x, y, z) *
    //                    ((MathExpression)this.Children[1]).Evaluate(x, y, z);
    //            case MathSymbol.Divide:
    //                return ((MathExpression)this.Children[0]).Evaluate(x, y, z) /
    //                    ((MathExpression)this.Children[1]).Evaluate(x, y, z);
    //            case MathSymbol.X:
    //                return x;
    //            case MathSymbol.Y:
    //                return y;
    //            case MathSymbol.Z:
    //                return z;
    //        }

    //        return double.NaN;
    //    }


    //    public override Expression<MathSymbol> CreateChild(MathSymbol symbol)
    //    {
    //        return new MathExpression(symbol, this);
    //    }

    //    public override int GetChildrenCapacity(MathSymbol symbol)
    //    {
    //        switch (symbol)
    //        {
    //            case MathSymbol.Add:
    //                return 2;
    //            case MathSymbol.Subtract:
    //                return 2;
    //            case MathSymbol.Multiply:
    //                return 2;
    //            case MathSymbol.Divide:
    //                return 2;
    //        }

    //        return 0;
    //    }
    //}

    public enum MathSymbol
    {
        Max = 7,
        FunctionMax = 4,
        Add = 0,
        Subtract = 1,
        Multiply = 2,
        Divide = 3,
        X = 4,
        Y = 5,
        Z = 6
    }
}
