﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DarwinDotNet;
using DarwinDotNet.Expressions;

namespace SymbolicRegression
{
    public static class GeneticsExtensions
    {
        public static double Evaluate(this Expression<MathSymbol> expression, double x, double y, 
            double z)
        {
            if (!expression.IsComplete)
                return double.NaN;

            switch (expression.Symbol)
            {
                case MathSymbol.Add:
                    return expression.Children[0].Evaluate(x, y, z) +
                        expression.Children[1].Evaluate(x, y, z);
                case MathSymbol.Subtract:
                    return expression.Children[0].Evaluate(x, y, z) -
                        expression.Children[1].Evaluate(x, y, z);
                case MathSymbol.Multiply:
                    return expression.Children[0].Evaluate(x, y, z) *
                        expression.Children[1].Evaluate(x, y, z);
                case MathSymbol.Divide:
                    return expression.Children[0].Evaluate(x, y, z) /
                        expression.Children[1].Evaluate(x, y, z);
                case MathSymbol.X:
                    return x;
                case MathSymbol.Y:
                    return y;
                case MathSymbol.Z:
                    return z;
            }

            return double.NaN;
        }

        public static char GetChar(this MathSymbol symbol)
        {
            switch (symbol)
            {
                case MathSymbol.Add:
                    return '+';
                case MathSymbol.Subtract:
                    return '-';
                case MathSymbol.Multiply:
                    return '*';
                case MathSymbol.Divide:
                    return '/';
                case MathSymbol.X:
                    return 'X';
                case MathSymbol.Y:
                    return 'Y';
                case MathSymbol.Z:
                    return 'Z';
                default:
                    throw new ArgumentOutOfRangeException("Unrecognised symbol.");
            }
        }
    }
}
